@@ -189,7 +189,6 @@ static void
 fm_icon_view_destroy (GtkObject *object)
 {
 	FMIconView *icon_view;
-	GtkUIManager *ui_manager;
 
 	icon_view = FM_ICON_VIEW (object);
 
@@ -206,13 +205,6 @@ fm_icon_view_destroy (GtkObject *object)
 		icon_view->details->icons_not_positioned = NULL;
 	}
 
-	ui_manager = fm_directory_view_get_ui_manager (FM_DIRECTORY_VIEW (icon_view));
-	if (ui_manager != NULL) {
-		nautilus_ui_unmerge_ui (ui_manager,
-					&icon_view->details->icon_merge_id,
-					&icon_view->details->icon_action_group);
-	}
-
 	GTK_OBJECT_CLASS (fm_icon_view_parent_class)->destroy (object);
 }
 
@@ -242,8 +234,7 @@ get_stored_icon_position_callback (NautilusIconContainer *container,
 				   FMIconView *icon_view)
 {
 	char *position_string, *scale_string;
-	gboolean position_good, scale_good;
-	char *locale;
+	gboolean position_good;
 	char c;
 
 	g_assert (NAUTILUS_IS_ICON_CONTAINER (container));
@@ -255,15 +246,6 @@ get_stored_icon_position_callback (NautilusIconContainer *container,
 		return FALSE;
 	}
 
-	/* Doing parsing in the "C" locale instead of the one set
-	 * by the user ensures that data in the metafile is not in
-	 * a locale-specific format. It's only necessary for floating
-	 * point values since there aren't locale-specific formats for
-	 * integers in C stdio.
-	 */
-	locale = g_strdup (setlocale (LC_NUMERIC, NULL));	
-	setlocale (LC_NUMERIC, "C");
-
 	/* Get the current position of this icon from the metadata. */
 	position_string = nautilus_file_get_metadata
 		(file, NAUTILUS_METADATA_KEY_ICON_POSITION, "");
@@ -277,17 +259,12 @@ get_stored_icon_position_callback (NautilusIconContainer *container,
 	/* Get the scale of the icon from the metadata. */
 	scale_string = nautilus_file_get_metadata
 		(file, NAUTILUS_METADATA_KEY_ICON_SCALE, "1");
-	scale_good = sscanf
-		(scale_string, " %lf",
-		 &position->scale) == 1;
-	if (!scale_good) {
-		position->scale = 1.0;
-	}
+	position->scale = g_ascii_strtod (scale_string, NULL);
+	if (errno != 0) {
+ 		position->scale = 1.0;
+ 	}
 
 	g_free (scale_string);
-
-	setlocale (LC_NUMERIC, locale);
-	g_free (locale);
 	
 	return position_good;
 }
@@ -1281,7 +1258,9 @@ fm_icon_view_set_zoom_level (FMIconView *view,
 
 	g_signal_emit_by_name (view, "zoom_level_changed");
 	
-	fm_directory_view_update_menus (FM_DIRECTORY_VIEW (view));
+	if (fm_directory_view_get_active (FM_DIRECTORY_VIEW (view))) {
+		fm_directory_view_update_menus (FM_DIRECTORY_VIEW (view));
+	}
 }
 
 static void
@@ -1646,6 +1625,24 @@ fm_icon_view_merge_menus (FMDirectoryView *view)
 	update_layout_menus (icon_view);
 }
 
+static void
+fm_icon_view_unmerge_menus (FMDirectoryView *view)
+{
+	FMIconView *icon_view;
+	GtkUIManager *ui_manager;
+
+	icon_view = FM_ICON_VIEW (view);
+
+	FM_DIRECTORY_VIEW_CLASS (fm_icon_view_parent_class)->unmerge_menus (view);
+
+	ui_manager = fm_directory_view_get_ui_manager (view);
+	if (ui_manager != NULL) {
+		nautilus_ui_unmerge_ui (ui_manager,
+					&icon_view->details->icon_merge_id,
+					&icon_view->details->icon_action_group);
+	}
+}
+
 static void
 fm_icon_view_update_menus (FMDirectoryView *view)
 {
@@ -1807,7 +1804,8 @@ icon_container_activate_callback (NautilusIconContainer *container,
 
 	fm_directory_view_activate_files (FM_DIRECTORY_VIEW (icon_view),
 					  file_list, 
-					  NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, 0);
+					  NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, 0,
+					  TRUE);
 }
 
 static void
@@ -1815,14 +1813,50 @@ icon_container_activate_alternate_callback (NautilusIconContainer *container,
 					    GList *file_list,
 					    FMIconView *icon_view)
 {
+	GdkEvent *event;
+	GdkEventButton *button_event;
+	GdkEventKey *key_event;
+	gboolean open_in_tab;
+	NautilusWindowInfo *window_info;
+	NautilusWindowOpenFlags flags;
+
 	g_assert (FM_IS_ICON_VIEW (icon_view));
 	g_assert (container == get_icon_container (icon_view));
 
+	open_in_tab = FALSE;
+
+	window_info = fm_directory_view_get_nautilus_window (FM_DIRECTORY_VIEW (icon_view));
+
+	if (nautilus_window_info_get_window_type (window_info) == NAUTILUS_WINDOW_NAVIGATION) {
+		event = gtk_get_current_event ();
+		if (event->type == GDK_BUTTON_PRESS ||
+		    event->type == GDK_BUTTON_RELEASE ||
+		    event->type == GDK_2BUTTON_PRESS ||
+		    event->type == GDK_3BUTTON_PRESS) {
+			button_event = (GdkEventButton *) event;
+			open_in_tab = (button_event->state & GDK_SHIFT_MASK) == 0;
+		} else if (event->type == GDK_KEY_PRESS ||
+			   event->type == GDK_KEY_RELEASE) {
+			key_event = (GdkEventKey *) event;
+			open_in_tab = !((key_event->state & GDK_SHIFT_MASK) != 0 &&
+				       (key_event->state & GDK_CONTROL_MASK) != 0);
+		} else {
+			open_in_tab = TRUE;
+		}
+	}
+
+	flags = NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND;
+	if (open_in_tab && eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ENABLE_TABS)) {
+		flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
+	} else {
+		flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
+	}
+
 	fm_directory_view_activate_files (FM_DIRECTORY_VIEW (icon_view), 
 					  file_list, 
 					  NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
-					  NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND |
-					  NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW);
+					  flags,
+					  TRUE);
 }
 
 static void
@@ -2063,8 +2097,8 @@ icon_container_preview_callback (NautilusIconContainer *container,
 			file_name = nautilus_file_get_display_name (file);
 			message = g_strdup_printf (_("pointing at \"%s\""), file_name);
 			g_free (file_name);
-			nautilus_window_info_set_status
-				(fm_directory_view_get_nautilus_window (FM_DIRECTORY_VIEW (icon_view)),
+			nautilus_window_slot_info_set_status
+				(fm_directory_view_get_nautilus_window_slot (FM_DIRECTORY_VIEW (icon_view)),
 				 message);
 			g_free (message);
 		} else {
@@ -2218,22 +2252,12 @@ icon_position_changed_callback (NautilusIconContainer *container,
 				FMIconView *icon_view)
 {
 	char *position_string;
-	char *scale_string;
-	char *locale;
+	char scale_string[G_ASCII_DTOSTR_BUF_SIZE];
 
 	g_assert (FM_IS_ICON_VIEW (icon_view));
 	g_assert (container == get_icon_container (icon_view));
 	g_assert (NAUTILUS_IS_FILE (file));
 
-	/* Doing formatting in the "C" locale instead of the one set
-	 * by the user ensures that data in the metafile is not in
-	 * a locale-specific format. It's only necessary for floating
-	 * point values since there aren't locale-specific formats for
-	 * integers in C stdio.
-	 */
-	locale = g_strdup (setlocale (LC_NUMERIC, NULL));	
-	setlocale (LC_NUMERIC, "C");
-
 	/* Schedule updating menus for the next idle. Doing it directly here
 	 * noticeably slows down icon stretching.  The other work here to
 	 * store the icon position and scale does not seem to noticeably
@@ -2241,7 +2265,8 @@ icon_position_changed_callback (NautilusIconContainer *container,
 	 * idle call, because we'd have to keep track of potentially multiple
 	 * sets of file/geometry info.
 	 */
-	if (icon_view->details->react_to_icon_change_idle_id == 0) {
+	if (fm_directory_view_get_active (FM_DIRECTORY_VIEW (icon_view)) &&
+	    icon_view->details->react_to_icon_change_idle_id == 0) {
                 icon_view->details->react_to_icon_change_idle_id
                         = g_idle_add (fm_icon_view_react_to_icon_change_idle_callback,
 				      icon_view);
@@ -2257,18 +2282,11 @@ icon_position_changed_callback (NautilusIconContainer *container,
 		g_free (position_string);
 	}
 
-	/* FIXME bugzilla.gnome.org 40662: 
-	 * %.2f is not a good format for the scale factor. We'd like it to
-	 * say "2" or "2x" instead of "2.00".
-	 */
-	scale_string = g_strdup_printf ("%.2f", position->scale);
+
+	g_ascii_dtostr (scale_string, sizeof (scale_string), position->scale);
 	nautilus_file_set_metadata
 		(file, NAUTILUS_METADATA_KEY_ICON_SCALE,
-		 "1.00", scale_string);
-	g_free (scale_string);
-
-	setlocale (LC_NUMERIC, locale);
-	g_free (locale);
+		 "1.0", scale_string);
 }
 
 /* Attempt to change the filename to the new text.  Notify user if operation fails. */
@@ -2758,6 +2776,7 @@ fm_icon_view_class_init (FMIconViewClass *klass)
         fm_directory_view_class->emblems_changed = fm_icon_view_emblems_changed;
         fm_directory_view_class->image_display_policy_changed = fm_icon_view_image_display_policy_changed;
         fm_directory_view_class->merge_menus = fm_icon_view_merge_menus;
+        fm_directory_view_class->unmerge_menus = fm_icon_view_unmerge_menus;
         fm_directory_view_class->sort_directories_first_changed = fm_icon_view_sort_directories_first_changed;
         fm_directory_view_class->start_renaming_file = fm_icon_view_start_renaming_file;
         fm_directory_view_class->text_attribute_names_changed = fm_icon_view_text_attribute_names_changed;
@@ -2877,22 +2896,28 @@ fm_icon_view_init (FMIconView *icon_view)
 }
 
 static NautilusView *
-fm_icon_view_create (NautilusWindowInfo *window)
+fm_icon_view_create (NautilusWindowSlotInfo *slot)
 {
 	FMIconView *view;
 
-	view = g_object_new (FM_TYPE_ICON_VIEW, "window", window, "compact", FALSE, NULL);
+	view = g_object_new (FM_TYPE_ICON_VIEW,
+			     "window-slot", slot,
+			     "compact", FALSE,
+			     NULL);
 	g_object_ref (view);
 	gtk_object_sink (GTK_OBJECT (view));
 	return NAUTILUS_VIEW (view);
 }
 
 static NautilusView *
-fm_compact_view_create (NautilusWindowInfo *window)
+fm_compact_view_create (NautilusWindowSlotInfo *slot)
 {
 	FMIconView *view;
 
-	view = g_object_new (FM_TYPE_ICON_VIEW, "window", window, "compact", TRUE, NULL);
+	view = g_object_new (FM_TYPE_ICON_VIEW,
+			     "window-slot", slot,
+			     "compact", TRUE,
+			     NULL);
 	g_object_ref (view);
 	gtk_object_sink (GTK_OBJECT (view));
 	return NAUTILUS_VIEW (view);
@@ -2966,7 +2991,12 @@ fm_icon_view_register (void)
 {
 	TRANSLATE_VIEW_INFO (fm_icon_view)
 	nautilus_view_factory_register (&fm_icon_view);
+}
 
+void
+fm_compact_view_register (void)
+{
 	TRANSLATE_VIEW_INFO (fm_compact_view)
 	nautilus_view_factory_register (&fm_compact_view);
 }
+
